;;;   calc.lsp
;;;  
;;;   Permission to use, copy, modify, and distribute this software and its
;;;   documentation for any purpose and without fee is hereby granted.  
;;;
;;;   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY. 
;;;   ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF 
;;;   MERCHANTABILITY ARE HEREBY DISCLAIMED.
;;; 
;;;   James Tunstall
;;;   Autodesk Australia
;;;   30 October 1991
;;;
;;;--------------------------------------------------------------------------;
;;;
;;;   *** User Guide ***
;;;
;;;   To use this program load this file.
;;;   i.e. (load "calc")
;;;
;;;   Then call up the calc function.
;;;
;;;   i.e. (calc)
;;;
;;;   Loading and/or calling up the function can be done at any
;;;   time within an AutoCAD command or at the command prompt.
;;;   The calculator will return a floating point value to AutoCAD
;;;   so it can be useful to use within commands requiring real
;;;   numbers as input.
;;;
;;;   Operation of the calculator is similar to a desktop calculator.
;;;
;;;   Start by typing the digits of a number.
;;;   N.B. DO NOT PRESS ENTER.
;;;
;;;   e.g. 26.3
;;;
;;;   If you make a mistake, the backspace or ESC key can be used.
;;;
;;;   Now hit a key to perform a specific function.
;;;
;;;   e.g. *
;;;
;;;   Type in another number.
;;;
;;;   e.g. 56.7
;;;
;;;   The result will be displayed when you press another function key.
;;;   You can press = to get the result immediately.
;;;   To exit the calculator press ENTER.
;;;   The result will be returned to AutoCAD and displayed on the
;;;   status line.
;;;
;;;   Function Keys:
;;;
;;;   Key               Action
;;;
;;;   =                 Display result
;;;   ESC               Cancel
;;;   BACKSPACE         Delete last character
;;;   TAB               Change sign
;;;   ENTER             Return result
;;;
;;;
;;;   Keys mapped to AutoLISP functions:
;;;
;;;   Key               AutoLISP function
;;;
;;;   ^                 expt
;;;   /                 /
;;;   *                 *
;;;   -                 -
;;;   +                 +
;;;   s                 sin1
;;;   alt-s             asin1
;;;   c                 cos1
;;;   alt-c             acos1
;;;   t                 tan1
;;;   alt-t             atan1
;;;   l                 log10
;;;   alt-l             alog10
;;;   r                 reciprocal
;;;   q                 sqr
;;;   alt-q             sqrt
;;;
;;;   All trigonometric functions accept and return the current system
;;;   unit for angles.
;;;
;;;--------------------------------------------------------------------------;
;;;
;;;   convert radians to current system unit for angles
;;;
(defun rtu (x)
   (if
      (or
         (= 0 (getvar "AUNITS"))
         (= 1 (getvar "AUNITS"))
         (= 4 (getvar "AUNITS"))
      )
      (setq x (/ (* x 180.0) pi))
   )
   (if (= 2 (getvar "AUNITS"))
      (setq x (/ (* x 200.0) pi))
   )
   ;; return converted angle
   x
)

;;;
;;;   convert current system unit for angles to radians
;;;
(defun utr (x)
   (if
      (or
         (= 0 (getvar "AUNITS"))
         (= 1 (getvar "AUNITS"))
         (= 4 (getvar "AUNITS"))
      )
      (setq x (* (/ x 180.0) pi))
   )
   (if (= 2 (getvar "AUNITS"))
      (setq x (* (/ x 200.0) pi))
   )
   ;; return converted angle
   x
)

;;;
;;;   square
;;;
(defun sqr ( x )
  (* x x)
)


;;;
;;;   log base 10
;;;
(defun log10 ( x )
   (/ (log x) (log 10.0))
)

;;;
;;;   anti log base 10
;;;
(defun alog10 ( x )
   (expt 10.0 x)
)

;;;
;;;   sine
;;;
(defun sin1 ( x )
   (sin (utr x))
)

;;;
;;;   anti sine
;;;
(defun asin1 ( x )
   (rtu (atan x (sqrt (- 1 (* x x)))))
)

;;;
;;;   cosine
;;;
(defun cos1 ( x )
   (cos (utr x))
)

;;;
;;;   anti cosine
;;;
(defun acos1 ( x )
   (rtu (atan (sqrt (- 1 (* x x))) x))
)

;;;
;;;   tan
;;;
(defun tan1 ( x )
   (/ (sin (utr x)) (cos (utr x)))
)

;;;
;;;   anti tan
;;;
(defun atan1 ( x )
   (rtu (atan x))
)

;;;
;;;   reciprocal
;;;
(defun reciprocal ( x )
   (/ 1.0 x)
)

;;;
;;;   list of calculator functions
;;;
(setq functions

   ;; list (<key> <weight> <function name>)

   (list
      (list 94 3 (quote expt))            ;^
      (list 47 2 (quote /))               ;/
      (list 42 2 (quote *))               ;*
      (list 45 1 (quote -))               ;-
      (list 43 1 (quote +))               ;+
      (list 115 0 (quote sin1))           ;s
      (list 83 0 (quote sin1))            ;S
      (list 159 0 (quote asin1))          ;alt-s
      (list 99 0 (quote cos1))            ;c
      (list 67 0 (quote cos1))            ;C
      (list 174 0 (quote acos1))          ;alt-c
      (list 116 0 (quote tan1))           ;t
      (list 84 0 (quote tan1))            ;T
      (list 148 0 (quote atan1))          ;alt-t
      (list 108 0 (quote log10))          ;l
      (list 76 0 (quote log10))           ;L
      (list 166 0 (quote alog10))         ;alt-l
      (list 114 0 (quote reciprocal))     ;r
      (list 82 0 (quote reciprocal))      ;R
      (list 113 0 (quote sqr))            ;q
      (list 81 0 (quote sqr))             ;Q
      (list 144 0 (quote sqrt))           ;alt-q
   )
)

;;;
;;;   tests to see if key is a function
;;;
(defun funtest ( key / function index )

   (setq function nil)
   (setq index 0)
   (while (< index (length functions))
      (if (= key (car (nth index functions)))
         (setq function index)
      )
      (setq index (1+ index))
   )

   ;; return which function
   function

)

;;;
;;;   tests to see if key is a digit
;;;
(defun isdigit ( key )
   (if
      (and
         (>= key 48)
         (<= key 57)
      )
      t
      nil
   )
)

;;;
;;; User defined error handling function
;;;
(defun calc_er (msg)

  (princ "error: ")                   ; display error message
  (princ msg)
  (terpri)

  (if olderr (setq *error* olderr))   ; restore old *error* function
  (princ)
)

;;;
;;;   main calculator function
;;;
(defun calc
   (
      / x y lastfun number decipt input len thisfun olderr
   )

   ;; Use own error function
   (setq olderr *error*)
   (setq *error* calc_er)

   (setq x 0.0)        ; first argument
   (setq y nil)        ; second argument
   (setq lastfun nil)  ; pointer to function in functions list

   (setq number "")    ; string holding numerical input
   (setq decipt nil)   ; if nil decimal point input allowed

   ;; display calculator status
   (grtext)
   (grtext
      -1
      (rtos x)
   )

   ;; get user input
   (setq input (grread))

   (while
      (and
         (= (car input) 2)             ; input must be from keyboard
         (/= (cadr input) 13)          ; carriage return terminates
      )
      (cond
         (
            ;; test for digit
            (isdigit (cadr input))
            (progn
               (setq number (strcat number (chr (cadr input))))
            )
         )
         (
            (and
               ;; test for decimal point
               (= (ascii ".") (cadr input))
               (null decipt)
            )
            (progn
               (setq number (strcat number "."))
               (setq decipt t)
            )
         )
         (
            (and
               ;; test for back space
               (= (cadr input) 8)
               (> (setq len (strlen number)) 0)
            )
            (progn
               ;; if last character decimal point
               ;; allow new decimal point in future input
               (if (= (substr number len 1) ".")
                  (setq decipt nil)
               )
               ;; delete last character
               (setq number (substr number 1 (- len 1)))
            )
         )
         (
            ;;test for tab
            (= (cadr input) 9)
            (if (> (strlen number) 0)
               ;; change sign
               (progn
                  (if (= "-" (substr number 1 1))
                     ;; strip negative sign
                     (setq number (substr number 2))
                     ;; add negative sign
                     (setq number (strcat "-" number))
                  )
               )
            )
         )
         (
            ;; test for escape
            (= (cadr input) 27)
            (progn
               ;; delete all characters
               (setq number "")
               (setq decipt nil)
            )
         )
         (
            ;; test for =
            (= (cadr input) 61)
            (progn
               (if (null y)
                  ;; no need for evaluation
                  (progn
                     (if (> (strlen number) 0)
                        (setq x (atof number))
                     )
                     (setq number "")
                     (setq decipt nil)
                  )
                  ;; evaluate result
                  (progn
                     (setq x
                        (apply
                           (caddr (nth lastfun functions))
                           (list
                              y
                              (if (= (strlen number) 0)
                                 x
                                 (atof number)
                              )
                           )
                        )
                     )
                     (setq number "")
                     (setq decipt nil)
                     (setq y nil)
                     (setq lastfun nil)
                  )
               )
            )
         )
         (
            (setq thisfun (funtest (cadr input)))
            (cond
               (
                  ;; if function weight is zero
                  (= (cadr (nth thisfun functions)) 0)
                  ;; apply this function to number
                  (progn
                     (setq x
                        (apply
                           (caddr (nth thisfun functions))
                           (list
                              (if (= (strlen number) 0)
                                 x
                                 (atof number)
                              )
                           )
                        )
                     )
                     (setq number "")
                     (setq decipt nil)
                  )
               )
               (
                  ;; if function weight > 0
                  (> (cadr (nth thisfun functions)) 0)
                  ;; test for second argument
                  (if (null y)
                     ;; no evaluation needed
                     (progn
                        (setq lastfun thisfun)
                        (if (> (strlen number) 0)
                           (setq y (atof number))
                           (setq y x)
                        )
                        (setq number "")
                        (setq decipt nil)
                     )
                     ;; apply last function to arguments
                     (progn
                        (setq x
                           (apply
                              (caddr (nth lastfun functions))
                              (list
                                 y
                                 (if (= (strlen number) 0)
                                    x
                                    (atof number)
                                 )
                              )
                           )
                        )
                        (setq number "")
                        (setq decipt nil)
                        (setq y x)
                        (setq lastfun thisfun)
                     )
                  )
               )
               (t nil)
            )
         )
         (t nil)
      )

      ;; display calculator status
      (grtext
         -1
         (if (= (strlen number) 0)
            (rtos x)
            number
         )
      )

      ;; get user input
      (setq input (grread))
   ); while loop

   ;; test for second argument
   (if (null y)
      ;; no evaluation required
      (progn
         (if (> (strlen number) 0)
            (setq x (atof number))
         )
      )
      ;; apply last function to arguments
      (progn
         (setq x
            (apply
               (caddr (nth lastfun functions))
               (list
                  y
                  (if (= (strlen number) 0)
                     x
                     (atof number)
                  )
               )
            )
         )
      )
   )
   ;; display result
   (grtext -1 (rtos x))

   (if olderr (setq *error* olderr))   ; restore old *error* function

   ;; return result
   x

)

